@@ -34,6 +34,9 @@
 
 struct _GConfSchema {
   GConfValueType type; /* Type of the described entry */
+  GConfValueType list_type; /* List type of the described entry */
+  GConfValueType car_type; /* Pair car type of the described entry */
+  GConfValueType cdr_type; /* Pair cdr type of the described entry */
   gchar* locale;       /* Schema locale */
   gchar* owner;        /* Name of creating application */
   gchar* short_desc;   /* 40 char or less description, no newlines */
@@ -46,6 +49,9 @@ void          gconf_schema_destroy(GConfSchema* sc);
 GConfSchema*  gconf_schema_copy(GConfSchema* sc);
 
 void          gconf_schema_set_type(GConfSchema* sc, GConfValueType type);
+void          gconf_schema_set_list_type(GConfSchema* sc, GConfValueType type);
+void          gconf_schema_set_car_type(GConfSchema* sc, GConfValueType type);
+void          gconf_schema_set_cdr_type(GConfSchema* sc, GConfValueType type);
 void          gconf_schema_set_locale(GConfSchema* sc, const gchar* locale);
 void          gconf_schema_set_short_desc(GConfSchema* sc, const gchar* desc);
 void          gconf_schema_set_long_desc(GConfSchema* sc, const gchar* desc);
@@ -54,6 +60,9 @@ void          gconf_schema_set_default_value(GConfSchema* sc, GConfValue* val);
 void          gconf_schema_set_default_value_nocopy(GConfSchema* sc, GConfValue* val);
 
 #define       gconf_schema_type(x) (x->type)
+#define       gconf_schema_list_type(x) (x->list_type)
+#define       gconf_schema_car_type(x) (x->car_type)
+#define       gconf_schema_cdr_type(x) (x->cdr_type)
 #define       gconf_schema_locale(x)     ((const gchar*)(x)->locale)
 #define       gconf_schema_short_desc(x) ((const gchar*)(x)->short_desc)
 #define       gconf_schema_long_desc(x)  ((const gchar*)(x)->long_desc)
