@@ -175,33 +175,329 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
   return value;
 }
 
+static char *
+escape_string(const char *str, const char *escaped_chars)
+{
+  gint i, j, len;
+  gchar* ret;
+
+  len = 0;
+  for (i = 0; str[i] != '\0'; i++)
+    {
+      if (strchr(escaped_chars, str[i]) != NULL ||
+	  str[i] == '\\')
+	len++;
+      len++;
+    }
+  
+  ret = g_malloc(len + 1);
+
+  j = 0;
+  for (i = 0; str[i] != '\0'; i++)
+    {
+      if (strchr(escaped_chars, str[i]) != NULL ||
+	  str[i] == '\\')
+	{
+	  ret[j++] = '\\';
+	}
+      ret[j++] = str[i];
+    }
+  ret[j++] = '\0';
+
+  return ret;
+}
+
 GConfValue*
 gconf_value_new_list_from_string(GConfValueType list_type,
-                                  const gchar* str)
+                                  const gchar* str,
+				  GConfError** err)
 {
+  int i, len;
+  gboolean escaped, pending_chars;
+  GString *string;
+  GConfValue* value;
+  GSList *list;
+
   g_return_val_if_fail(list_type != GCONF_VALUE_LIST, NULL);
   g_return_val_if_fail(list_type != GCONF_VALUE_PAIR, NULL);
 
-  /* FIXME (parse the string generated by gconf_value_to_string();
-     probably need to add string escaping to that function to make
-     lists/pairs of string work */
-  
-  return NULL;
+  if (str[0] != '[')
+    {
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (list must start with a '[')"),
+			       str);
+      return NULL;
+    }
+
+  len = strlen(str);
+
+  /* Note: by now len is sure to be 1 or larger, so len-1 will never be
+   * negative */
+  if (str[len-1] != ']')
+    {
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (list must end with a ']')"),
+			       str);
+      return NULL;
+    }
+
+  if (strstr(str, "[]"))
+    {
+      value = gconf_value_new(GCONF_VALUE_LIST);
+      gconf_value_set_list_type(value, list_type);
+
+      return value;
+    }
+
+  escaped = FALSE;
+  pending_chars = FALSE;
+  list = NULL;
+  string = g_string_new(NULL);
+
+  for (i = 1; str[i] != '\0'; i++)
+    {
+      if ( ! escaped &&
+	  (str[i] == ',' ||
+	   str[i] == ']'))
+	{
+	  GConfValue* val;
+	  val = gconf_value_new_from_string(list_type, string->str, err);
+
+	  if (err && *err != NULL)
+	    {
+	      /* Free values so far */
+	      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+	      g_slist_free(list);
+
+	      g_string_free(string, TRUE);
+
+	      return NULL;
+	    }
+
+	  g_string_assign(string, "");
+	  list = g_slist_prepend(list, val);
+	  if (str[i] == ']' &&
+	      i != len-1)
+	    {
+	      /* Free values so far */
+	      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+	      g_slist_free(list);
+
+	      g_string_free(string, TRUE);
+
+	      if (err)
+		*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+				       _("Didn't understand `%s' (extra unescaped ']' found inside list)"),
+				       str);
+	      return NULL;
+	    }
+	  pending_chars = FALSE;
+	}
+      else if ( ! escaped && str[i] == '\\')
+	{
+	  escaped = TRUE;
+	  pending_chars = TRUE;
+	}
+      else
+	{
+	  g_string_append_c(string, str[i]);
+	  escaped = FALSE;
+	  pending_chars = TRUE;
+	}
+    }
+
+  g_string_free(string, TRUE);
+
+  if (pending_chars)
+    {
+      /* Free values so far */
+      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+      g_slist_free(list);
+
+      g_string_free(string, TRUE);
+
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (extra trailing characters)"),
+			       str);
+      return NULL;
+    }
+
+  /* inverse list as we were prepending to it */
+  list = g_slist_reverse(list);
+
+  value = gconf_value_new(GCONF_VALUE_LIST);
+  gconf_value_set_list_type(value, list_type);
+
+  gconf_value_set_list_nocopy(value, list);
+
+  return value;
 }
 
 GConfValue*
 gconf_value_new_pair_from_string(GConfValueType car_type,
                                   GConfValueType cdr_type,
-                                  const gchar* str)
+                                  const gchar* str,
+				  GConfError** err)
 {
+  int i, len;
+  int elem;
+  gboolean escaped, pending_chars;
+  GString *string;
+  GConfValue* value;
+  GConfValue* car;
+  GConfValue* cdr;
+
   g_return_val_if_fail(car_type != GCONF_VALUE_LIST, NULL);
   g_return_val_if_fail(car_type != GCONF_VALUE_PAIR, NULL);
+  g_return_val_if_fail(cdr_type != GCONF_VALUE_LIST, NULL);
+  g_return_val_if_fail(cdr_type != GCONF_VALUE_PAIR, NULL);
 
-  /* FIXME (parse the string generated by gconf_value_to_string();
-     probably need to add string escaping to that function to make
-     lists/pairs of string work */
+  if (str[0] != '(')
+    {
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (pair must start with a '(')"),
+			       str);
+      return NULL;
+    }
 
-  return NULL;
+  len = strlen(str);
+
+  /* Note: by now len is sure to be 1 or larger, so len-1 will never be
+   * negative */
+  if (str[len-1] != ')')
+    {
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (pair must end with a ')')"),
+			       str);
+      return NULL;
+    }
+
+  escaped = FALSE;
+  pending_chars = FALSE;
+  car = cdr = NULL;
+  string = g_string_new(NULL);
+  elem = 0;
+
+  for (i = 1; str[i] != '\0'; i++)
+    {
+      if ( ! escaped &&
+	  (str[i] == ',' ||
+	   str[i] == ')'))
+	{
+	  if ((str[i] == ')' && elem != 1) ||
+	      (elem > 1))
+	    {
+	      /* Free values so far */
+	      if (car)
+	        gconf_value_destroy(car);
+	      if (cdr)
+	        gconf_value_destroy(cdr);
+
+	      g_string_free(string, TRUE);
+
+	      if (err)
+		*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+				       _("Didn't understand `%s' (wrong number of elements)"),
+				       str);
+	      return NULL;
+	    }
+
+	  if (elem == 0)
+	    car = gconf_value_new_from_string(car_type, string->str, err);
+	  else if (elem == 1)
+	    cdr = gconf_value_new_from_string(cdr_type, string->str, err);
+
+	  elem ++;
+
+	  if (err && *err != NULL)
+	    {
+	      /* Free values so far */
+	      if (car)
+	        gconf_value_destroy(car);
+	      if (cdr)
+	        gconf_value_destroy(cdr);
+
+	      g_string_free(string, TRUE);
+
+	      return NULL;
+	    }
+
+	  g_string_assign(string, "");
+
+	  if (str[i] == ')' &&
+	      i != len-1)
+	    {
+	      /* Free values so far */
+	      if (car)
+	        gconf_value_destroy(car);
+	      if (cdr)
+	        gconf_value_destroy(cdr);
+
+	      g_string_free(string, TRUE);
+
+	      if (err)
+		*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+				       _("Didn't understand `%s' (extra unescaped ')' found inside pair)"),
+				       str);
+	      return NULL;
+	    }
+	  pending_chars = FALSE;
+	}
+      else if ( ! escaped && str[i] == '\\')
+	{
+	  escaped = TRUE;
+	  pending_chars = TRUE;
+	}
+      else
+	{
+	  g_string_append_c(string, str[i]);
+	  escaped = FALSE;
+	  pending_chars = TRUE;
+	}
+    }
+
+  g_string_free(string, TRUE);
+
+  if (pending_chars)
+    {
+      /* Free values so far */
+      if (car)
+	gconf_value_destroy(car);
+      if (cdr)
+	gconf_value_destroy(cdr);
+
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (extra trailing characters)"),
+			       str);
+      return NULL;
+    }
+
+  if (elem != 2)
+    {
+      /* Free values so far */
+      if (car)
+	gconf_value_destroy(car);
+      if (cdr)
+	gconf_value_destroy(cdr);
+
+      if (err)
+	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
+			       _("Didn't understand `%s' (wrong number of elements)"),
+			       str);
+      return NULL;
+    }
+
+  value = gconf_value_new(GCONF_VALUE_PAIR);
+  gconf_value_set_car_nocopy(value, car);
+  gconf_value_set_cdr_nocopy(value, cdr);
+
+  return value;
 }
 
 gchar*
@@ -252,12 +548,17 @@ gconf_value_to_string(GConfValue* value)
             
             while (list != NULL)
               {
+                gchar* tmp;
                 gchar* elem;
                 guint len;
                 
-                elem = gconf_value_to_string((GConfValue*)list->data);
+                tmp = gconf_value_to_string((GConfValue*)list->data);
 
-                g_assert(elem != NULL);
+                g_assert(tmp != NULL);
+
+		elem = escape_string(tmp, ",]");
+
+		g_free(tmp);
 
                 len = strlen(elem);
 
@@ -295,12 +596,17 @@ gconf_value_to_string(GConfValue* value)
       break;
     case GCONF_VALUE_PAIR:
       {
+        gchar* tmp;
         gchar* car;
         gchar* cdr;
 
-        car = gconf_value_to_string(gconf_value_car(value));
-        cdr = gconf_value_to_string(gconf_value_cdr(value));
-        retval = g_strdup_printf("(%s, %s)", car, cdr);
+        tmp = gconf_value_to_string(gconf_value_car(value));
+	car = escape_string(tmp, ",)");
+	g_free(tmp);
+        tmp = gconf_value_to_string(gconf_value_cdr(value));
+	cdr = escape_string(tmp, ",)");
+	g_free(tmp);
+        retval = g_strdup_printf("(%s,%s)", car, cdr);
         g_free(car);
         g_free(cdr);
       }
@@ -313,12 +619,20 @@ gconf_value_to_string(GConfValue* value)
       {
         const gchar* locale;
         const gchar* type;
+        const gchar* list_type;
+        const gchar* car_type;
+        const gchar* cdr_type;
         
         locale = gconf_schema_locale(gconf_value_schema(value));
         type = gconf_value_type_to_string(gconf_schema_type(gconf_value_schema(value)));
+        list_type = gconf_value_type_to_string(gconf_schema_list_type(gconf_value_schema(value)));
+        car_type = gconf_value_type_to_string(gconf_schema_car_type(gconf_value_schema(value)));
+        cdr_type = gconf_value_type_to_string(gconf_schema_cdr_type(gconf_value_schema(value)));
         
-        retval = g_strdup_printf("Schema (type: `%s' locale: `%s')",
-                                 type, locale ? locale : "(null)");
+        retval = g_strdup_printf("Schema (type: `%s' list_type: '%s' "
+				 "car_type: '%s' cdr_type: '%s' locale: `%s')",
+                                 type, list_type, car_type, cdr_type,
+				 locale ? locale : "(null)");
       }
       break;
     default:
