@@ -113,6 +113,10 @@ enum {
   LAST_SIGNAL
 };
 
+static void         register_client   (GConfClient *client);
+static GConfClient *lookup_client     (GConfEngine *engine);
+static void         unregister_client (GConfClient *client);
+
 static void gconf_client_class_init (GConfClientClass *klass);
 static void gconf_client_init       (GConfClient      *client);
 static void gconf_client_real_unreturned_error (GConfClient* client, GConfError* error);
@@ -132,7 +136,8 @@ static gboolean gconf_client_lookup         (GConfClient* client,
                                              GConfValue** valp);
 
 static void gconf_client_real_remove_dir    (GConfClient* client,
-                                             Dir* d);
+                                             Dir* d,
+					     GConfError** err);
 
 static guint client_signals[LAST_SIGNAL] = { 0 };
 static GtkObjectClass* parent_class = NULL;
@@ -224,7 +229,8 @@ destroy_dir_foreach_remove(gpointer key, gpointer value, gpointer user_data)
   
   /* remove notify for this dir */
   
-  gconf_notify_remove(client->engine, d->notify_id);
+  if(d->notify_id != 0)
+	  gconf_notify_remove(client->engine, d->notify_id);
   d->notify_id = 0;
 
   dir_destroy(value);
@@ -262,6 +268,8 @@ gconf_client_finalize (GtkObject* object)
   
   g_hash_table_destroy(client->cache_hash);
   client->cache_hash = NULL;
+
+  unregister_client (client);
   
   if (client->engine != NULL)
     {
@@ -408,54 +416,125 @@ notify_from_server_callback(GConfEngine* conf, guint cnxn_id,
  */
 
 GConfClient*
-gconf_client_new (void)
+gconf_client_get_default (void)
 {
   GConfClient *client;
-
-  g_return_val_if_fail(gconf_is_initialized(), NULL);
+  GConfEngine *engine;
   
-  client = gtk_type_new (gconf_client_get_type ());
+  g_return_val_if_fail(gconf_is_initialized(), NULL);
 
-  client->engine = gconf_engine_new();
+  engine = gconf_engine_get_default ();
+  
+  client = lookup_client (engine);
+  if (client)
+    {
+      g_assert (client->engine == engine);
+      gtk_object_ref (GTK_OBJECT (client));
+      gconf_engine_unref (engine);
+      return client;
+    }
+  else
+    {
+      client = gtk_type_new (gconf_client_get_type ());
+
+      /* Emulate GObject */
+      gtk_object_ref (GTK_OBJECT (client));
+      gtk_object_sink (GTK_OBJECT (client));
+
+      client->engine = engine;
+      register_client (client);
+    }
   
   return client;
 }
 
 GConfClient*
-gconf_client_new_with_engine (GConfEngine* engine)
+gconf_client_get_for_engine (GConfEngine* engine)
 {
   GConfClient *client;
 
   g_return_val_if_fail(gconf_is_initialized(), NULL);
-  
-  client = gtk_type_new (gconf_client_get_type ());
 
-  client->engine = engine;
+  client = lookup_client (engine);
+  if (client)
+    {
+      g_assert (client->engine == engine);
+      gtk_object_ref (GTK_OBJECT (client));
+      return client;
+    }
+  else
+    {
+      client = gtk_type_new (gconf_client_get_type ());
 
-  gconf_engine_ref(client->engine);
+      /* Emulate GObject */
+      gtk_object_ref (GTK_OBJECT (client));
+      gtk_object_sink (GTK_OBJECT (client));
+      
+      client->engine = engine;
+
+      gconf_engine_ref(client->engine);
+
+      register_client (client);
+    }
   
   return client;
 }
 
+typedef struct {
+  GConfClient *client;
+  Dir *lower_dir;
+  const char *dirname;
+} OverlapData;
 
+static void
+foreach_setup_overlap(gpointer key, gpointer value, gpointer user_data)
+{
+  GConfClient *client;
+  Dir *dir = value;
+  OverlapData * od = user_data;
+
+  client = od->client;
+
+  /* if we have found the first (well there is only one anyway) directory
+   * that includes us that has a notify handler */
 #ifdef GCONF_ENABLE_DEBUG
-static void
-foreach_check_overlap(gpointer key, gpointer value, gpointer user_data)
-{
-  /* Disallow overlap */
-  g_return_if_fail(!gconf_key_is_below(key, user_data));
-  g_return_if_fail(!gconf_key_is_below(user_data, key));
+  if (dir->notify_id != 0 &&
+      gconf_key_is_below(dir->name, od->dirname))
+    {
+      g_assert(od->lower_dir == NULL);
+      od->lower_dir = dir;
+    }
+#else
+  if (od->lower_dir == NULL &&
+      dir->notify_id != 0 &&
+      gconf_key_is_below(dir->name, od->dirname))
+      od->lower_dir = dir;
+#endif
+  /* if we have found a directory that we include and it has
+   * a notify_id, remove the notify handler now
+   * FIXME: this is a race, from now on we can miss notifies, it is
+   * not an incredible amount of time so this is not a showstopper */
+  else if (dir->notify_id != 0 &&
+	   gconf_key_is_below(od->dirname, dir->name))
+    {
+      gconf_notify_remove(client->engine, dir->notify_id);
+      dir->notify_id = 0;
+    }
 }
 
-static void
-check_overlap(GConfClient* client, const gchar* dirname)
+static Dir *
+setup_overlaps(GConfClient* client, const gchar* dirname)
 {
-  g_hash_table_foreach(client->dir_hash, foreach_check_overlap,
-                       (gchar*)dirname);
+  OverlapData od;
+
+  od.client = client;
+  od.lower_dir = NULL;
+  od.dirname = dirname;
+
+  g_hash_table_foreach(client->dir_hash, foreach_setup_overlap, &od);
+
+  return od.lower_dir;
 }
-#else
-#define check_overlap(x,y)
-#endif
 
 void
 gconf_client_add_dir     (GConfClient* client,
@@ -473,23 +552,35 @@ gconf_client_add_dir     (GConfClient* client,
 
   if (d == NULL)
     {
-      check_overlap(client, dirname);
+      Dir *overlap_dir;
 
-      notify_id = gconf_notify_add(client->engine,
-                                   dirname,
-                                   notify_from_server_callback,
-                                   client,
-                                   &error);
-      
-      /* We got a notify ID or we got an error, not both */
-      g_return_if_fail( (notify_id != 0 && error == NULL) ||
-                        (notify_id == 0 && error != NULL) );
-      
-      
-      if (handle_error(client, error, err))
-        return;
+      overlap_dir = setup_overlaps(client, dirname);
 
-      g_assert(error == NULL);
+      /* only if there is no directory that includes us
+       * already add a notify */
+      if (overlap_dir == NULL)
+        {
+
+          notify_id = gconf_notify_add(client->engine,
+                                       dirname,
+                                       notify_from_server_callback,
+                                       client,
+                                       &error);
+      
+          /* We got a notify ID or we got an error, not both */
+          g_return_if_fail( (notify_id != 0 && error == NULL) ||
+                            (notify_id == 0 && error != NULL) );
+      
+      
+          if (handle_error(client, error, err))
+            return;
+
+          g_assert(error == NULL);
+	}
+      else
+	{
+	  notify_id = 0;
+	}
       
       d = dir_new(dirname, notify_id);
 
@@ -505,10 +596,55 @@ gconf_client_add_dir     (GConfClient* client,
   d->add_count += 1;
 }
 
+typedef struct {
+  GConfClient *client;
+  GConfError *error;
+} AddNotifiesData;
+
+static void
+foreach_add_notifies(gpointer key, gpointer value, gpointer user_data)
+{
+  AddNotifiesData *ad = user_data;
+  GConfClient *client;
+  Dir *dir = value;
+
+  client = ad->client;
+
+  if (ad->error != NULL)
+    return;
+
+  if (dir->notify_id == 0)
+    {
+      Dir *overlap_dir;
+      overlap_dir = setup_overlaps(client, dir->name);
+
+      /* only if there is no directory that includes us
+       * already add a notify */
+      if (overlap_dir == NULL)
+        {
+          dir->notify_id = gconf_notify_add(client->engine,
+					    dir->name,
+					    notify_from_server_callback,
+					    client,
+					    &ad->error);
+
+          /* We got a notify ID or we got an error, not both */
+          g_return_if_fail( (dir->notify_id != 0 && ad->error == NULL) ||
+		            (dir->notify_id == 0 && ad->error != NULL) );
+
+	  /* if error is returned, then we'll just ignore
+	   * things until the end */
+	}
+    }
+}
+
 static void
 gconf_client_real_remove_dir    (GConfClient* client,
-                                 Dir* d)
+                                 Dir* d,
+				 GConfError** err)
 {
+  AddNotifiesData ad;
+
   g_return_if_fail(d != NULL);
   g_return_if_fail(d->add_count == 0);
   
@@ -516,15 +652,24 @@ gconf_client_real_remove_dir    (GConfClient* client,
   
   /* remove notify for this dir */
   
-  gconf_notify_remove(client->engine, d->notify_id);
+  if(d->notify_id != 0)
+	  gconf_notify_remove(client->engine, d->notify_id);
   d->notify_id = 0;
   
   dir_destroy(d);
+
+  ad.client = client;
+  ad.error = NULL;
+
+  g_hash_table_foreach(client->dir_hash, foreach_add_notifies, &ad);
+
+  handle_error(client, ad.error, err);
 }
 
 void
 gconf_client_remove_dir  (GConfClient* client,
-                          const gchar* dirname)
+                          const gchar* dirname,
+			  GConfError** err)
 {
   Dir* found = NULL;
 
@@ -538,7 +683,7 @@ gconf_client_remove_dir  (GConfClient* client,
       found->add_count -= 1;
 
       if (found->add_count == 0) 
-        gconf_client_real_remove_dir(client, found);
+        gconf_client_real_remove_dir(client, found, err);
     }
 #ifndef G_DISABLE_CHECKS
   else
@@ -1089,7 +1234,8 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
       g_assert(error == NULL);
       
       if (check_type(key, val, GCONF_VALUE_STRING, &error))
-        retval = g_strdup(gconf_value_string(val));
+	/* we cheat here (look below) so we have to cast this */
+        retval = (gchar *)gconf_value_string(val);
       else
         handle_error(client, error, err);
 
@@ -1415,7 +1561,7 @@ gconf_client_set_pair    (GConfClient* client, const gchar* key,
   g_return_val_if_fail(key != NULL, FALSE);
   
   if (gconf_set_pair(client->engine, key, car_type, cdr_type,
-                     address_of_car, address_of_car, &error))
+                     address_of_car, address_of_cdr, &error))
     return TRUE;
   else
     {
@@ -1888,3 +2034,31 @@ gconf_client_create_change_set_from_current (GConfClient* client,
 
   return retval;
 }
+
+static GHashTable * clients = NULL;
+
+static void
+register_client (GConfClient *client)
+{
+  if (clients == NULL)
+    clients = g_hash_table_new (NULL, NULL);
+
+  g_hash_table_insert (clients, client->engine, client);
+}
+
+static GConfClient *
+lookup_client (GConfEngine *engine)
+{
+  if (clients == NULL)
+    return NULL;
+  else
+    return g_hash_table_lookup (clients, engine);
+}
+
+static void
+unregister_client (GConfClient *client)
+{
+  g_return_if_fail (clients != NULL);
+
+  g_hash_table_remove (clients, client->engine);
+}
